Attribute VB_Name = "API_ErrorStrings"
' ------------------------------------------------------
' API_Err.BAS -- Copyright (c) Slightly Tilted Software
' By: L.J. Johnson       Date: 06-15-1996
' Comments:    Contains ReturnApiErrString()
' ------------------------------------------------------
Option Explicit


' -------------------------------------------------
' Used to get error messages directly from the
'    system instead of hard-coding them
' -------------------------------------------------
Private Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Private Const FORMAT_MESSAGE_IGNORE_INSERTS = &H200

Private Declare Function FormatMessage Lib "kernel32" Alias _
    "FormatMessageA" _
      (ByVal dwFlags As Long, lpSource As Any, _
       ByVal dwMessageId As Long, _
       ByVal dwLanguageId As Long, _
       ByVal lpBuffer As String, _
       ByVal nSize As Long, _
       Arguments As Long) As Long


' -------------------------------------------------
' Custom error return codes for the
'   RegisterEvntSource class module
' -------------------------------------------------
Public Const ERR_NO_CREATE_KEY = 1011
Public Const ERR_NO_OPEN_KEY = 1012
Public Const ERR_NO_SET_FIRST_VALUE = 1013
Public Const ERR_NO_SET_SECOND_VALUE = 1014
Public Const ERR_NO_CLOSE_KEY = 1015


' -------------------------------------------------
' Custom error return codes for the AppEventLog
'   class module
' -------------------------------------------------
Public Const ERR_REG_EVENT_SOURCE = 1111
Public Const ERR_DEREG_EVENT_SOURCE = 1112
Public Const ERR_REPORT_EVENT = 1113


' --------------------------------------------------------
' Status Codes
' --------------------------------------------------------
'Public Const INVALID_HANDLE_VALUE = -1&
Public Const ERROR_SUCCESS = 0&


' -------------------------------------------------
' EVENTLOG.BAS / ReturnApiErrString
'
' Passed an API error number, return an error
'   string
'
' Comments:    Takes an API error number, and returns
'              a descriptive text string of the error
'       Inputs:   xlngError is the number returned from
'                    the API error
'      Outputs:   Function returns the error string
'
' The original code appeared in Keith Pleas' article
'   in VBPJ, April 1996 (OLE Expert column).  Thanks,
'   Keith.
' -------------------------------------------------
Public Function ReturnApiErrString(ErrorCode As Long) As String
   Dim pstrBuffer    As String

   ' ----------------------------------------------
   ' Allocate the string, then get the system to
   '   tell us the error message associated with
   '   this error number
   ' ----------------------------------------------
   pstrBuffer = String(256, 0)
   FormatMessage FORMAT_MESSAGE_FROM_SYSTEM _
      Or FORMAT_MESSAGE_IGNORE_INSERTS, 0&, _
      ErrorCode, 0&, pstrBuffer, Len(pstrBuffer), 0&


   ' ----------------------------------------------
   ' Strip the last null, then the last CrLf pair if
   ' it exists
   ' ----------------------------------------------
   pstrBuffer = Left(pstrBuffer, InStr(pstrBuffer, vbNullChar) - 1)
   If Right$(pstrBuffer, 2) = Chr$(13) & Chr$(10) Then
      pstrBuffer = Mid$(pstrBuffer, 1, Len(pstrBuffer) - 2)
   End If


   ' ----------------------------------------------
   ' Set the return value
   ' ----------------------------------------------
   ReturnApiErrString = pstrBuffer

End Function




